#include <libporty/net/network.h>

/* start a resolver in the background */
int P_net_async_dns(P_net_socket *fd);

/* ---- helper functions that can do the protocol parsing ----- */
typedef enum P_net_async_dns_outcome_e {
	P_NET_DNS_OUT_OK        = 0, /* no error */
	P_NET_DNS_OUT_PROTOCOL  = 1,
	P_NET_DNS_OUT_NUMARGS   = 2,
	P_NET_DNS_OUT_COMMAND   = 3,
	P_NET_DNS_OUT_RESOLV    = 4,
	P_NET_DNS_OUT_INTERNAL  = -1
} P_net_async_dns_outcome_t;

typedef struct P_net_async_dns_req_s {
	union {
		void *p;
		long int i;
		double d;
		char s[32];    /* works until we have 256 bit platforms */
	} cookie;

	/* Either of these should be NULL for a request */
	char *hostname;
	char *ip_addr;

	/* outcome of the action - error if non-zero */
	P_net_async_dns_outcome_t outcome;

	/* later may want to have a protocol field here, at the end */
	int reserved1;
} P_net_async_dns_req_t;

/* send a query request to a background resolver */
int P_net_async_dns_req(P_net_socket fd, P_net_async_dns_req_t *req);

/* read answer(s) from the comm socket (*buf should be NULL for the first call) */
int P_net_async_dns_read(void **buf, P_net_socket fd);

/* return next parsed answer from buffer, into ans */
int P_net_async_dns_parse(void **buf, P_net_async_dns_req_t *ans);

/* destroy a background resolver */
int P_net_async_dns_destroy(void **buf, P_net_socket fd);
