/* Unlike other sources of this project, this specific example application
   is placed in the public domain. The purpose is to ease copy&pasting
   code from the example to a real project. */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "gentrex.h"
#include "helper_map_az.h"
#include "test_tree.h"
#include "test_graphviz.h"

int map_print[26];
gentrex_map_az_t map, map_init;

int main(int argc, char *argv[])
{
	int dump = 0;
	gentrex_state_t *st;
	gentrex_api_t api;
	int n, match;
	char *res, *s;
	char *dot_first_name = NULL;
	char *dot_all_name = NULL;
	char script[1024];
	const char *err;

	/* init the example tree */
	tree_init();

	/* parse command line args */
	for(n = 1; n < argc; n++) {
		char *cmd = argv[n];
		if (cmd[0] == '-') {
			switch(cmd[1]) {
				case 'd': dump = 1; break;
				case 'g': n++; dot_first_name = argv[n]; break;
				case 'G': n++; dot_all_name = argv[n]; break;
				case 'p':
					n++;
					for(s = argv[n]; *s != '\0'; s++)
						if ((*s >= 'a') && (*s <= 'z'))
							map_print[*s - 'a'] = 1;
			}
		}
		else if ((cmd[0] != '\0') && (cmd[1] == '=')) {
			cmd[1] = '\0';
			if ((cmd[0] >= 'a') && (cmd[0] <= 'z'))
				gentrex_map_az_set_node(&map_init, cmd, (char **)tree_user + T(cmd[2]));
			else if ((cmd[0] >= 'A') && (cmd[0] <= 'Z'))
				tree_user[cmd[0] - 'A'] = cmd+2;
			else
				goto unrec;
		}
		else {
			unrec:;
			fprintf(stderr, "Unrecognized arg: '%s'\n", cmd);
			exit(1);
		}
	}

	/* set up the API using the example tree and the simplistic a-z map */
	memset(&api, 0, sizeof(api));
	api.user_map_ctx = &map;
	api.map_get = gentrex_map_az_get;
	api.map_set = gentrex_map_az_set;

	api.next   = tree_next;
	api.child  = tree_child;
	api.parent = tree_parent;

	api.tag_eq    = tag_eq;
	api.tag_match = tag_eq;

	/* read the first non-empty line from stdin and use that as an expression */
	while(!(feof(stdin))) {
		char *s;
		*script = '\0';
		fgets(script, sizeof(script), stdin);
		s = script;
		while(isspace(*s)) s++;
		if (*s != '\0') {
			st = gentrex_alloc(&api, s, &err);
			break;
		}
	}

	if (st == NULL) {
		fprintf(stderr, "Error: %s\n", err);
		return 1;
	}

	if (dump) {
		gentrex_dump(st, tree_user);
		printf("\n");
	}

	/* iterate over all matches */
	for(match=0;;match++) {

		/* restore the initial map for each subsequent query */
		memcpy(&map, &map_init, sizeof(map));

		/* eval the expression, get the next match */
		res = gentrex_execute(st, tree_user);
		if (res == NULL)
			break;
		tree_print_node("execute: ", res);

		/* print all variables from the map that are requested with -p on the CLI */
		for(n = 0; n < 26; n++) {
			if (map_print[n]) {
				char name[2];
				printf("      %c: ", n+'a');
				name[0] = n+'a';
				name[1] = '\0';
				tree_print_node("", gentrex_map_az_get_node(&map, name));
			}
		}

		if ((match == 0) && (dot_first_name != NULL))
			graphviz_dot_first(&map, res, dot_first_name);

		if (dot_all_name != NULL)
			graphviz_save(&map, res);

		printf("\n");
	};

	if (dot_all_name != NULL)
		graphviz_dot_all(&map, dot_all_name);

	gentrex_free(st);

	return 0;
}
