/*  gentrex - generic tree expressions; generate graphviz scripts from search results
    Copyright (C) 2015  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/gentrex
    Contact the author (email and IRC): http://igor2.repo.hu/contact.html
*/
#include <stdlib.h>
#include <stdio.h>
#include "test_graphviz.h"

static void gv_draw_edges(FILE *f)
{
	int n;
	for(n = 1; n < 26; n++) {
		if (tree_parent_tab[n] >= 0) {
			fprintf(f, "\t%c -> %c\n", 'A'+tree_parent_tab[n], 'A'+n);
		}
	}
}

void graphviz_dot_first(gentrex_map_az_t *map, void *res, const char *fn)
{
	FILE *f;
	int n;

	f = fopen(fn, "w");
	fprintf(f, "digraph tree {\n");
	for(n = 0; n < 26; n++) {
		int m, has_map = 0;
		char name = n+'A';
		char *user = tree_user[n];
		fprintf(f, "\t%c [label=\"%c\\n%s", name, name, (user == NULL) ? "" : user);
		for(m = 0; m < 26; m++) {
			if (map->map[m].straight == &tree_user[n]) {
				if (has_map == 0)
					fprintf(f,"\\n");
				else
					fprintf(f,", ");
				fprintf(f, "%c", m + 'a');
				has_map++;
			}
		}
		fprintf(f, "\"");
		if (res == tree_user+n)
			fprintf(f, " color=red");
		if (has_map)
			fprintf(f, " shape=box");
		fprintf(f, "]\n");
	}

	gv_draw_edges(f);
	fprintf(f, "}\n");
	fclose(f);
}


/* node->map reference count indexed by [node_id][map_id] */
static int gv_map[26][26];

/* count how many times a given node was a result */
static int gv_res[26];

void graphviz_save(gentrex_map_az_t *map, void *res)
{
	int m;
	for(m = 0; m < 26; m++) {
		if (map->map[m].straight != NULL) {
			int node = (char **)map->map[m].straight - (char **)tree_user;
			gv_map[node][m]++;
		}
	}

	if (res != NULL) {
		int node = (char **)res - (char **)tree_user;
		gv_res[node]++;
	}
}

void graphviz_dot_all(gentrex_map_az_t *map, const char *fn)
{
	FILE *f;
	int n, m, has_map = 0;

	f = fopen(fn, "w");
	fprintf(f, "digraph tree {\n");

	/* print normal nodes */
	for(n = 0; n < 26; n++) {
		char name = n+'A';
		char *user = tree_user[n];
		fprintf(f, "\t%c [label=\"%c\\n%s", name, name, (user == NULL) ? "" : user);
		has_map = 0;
		for(m = 0; m < 26; m++) {
			if (gv_map[n][m] > 0) {
				if (has_map == 0)
					fprintf(f,"\\n");
				else
					fprintf(f,", ");
				fprintf(f, "%c(%d)", m + 'a', gv_map[n][m]);
				has_map++;
			}
		}
		if (gv_res[n] > 0)
			fprintf(f, "\\n*%d", gv_res[n]);
		fprintf(f, "\"");
		if (gv_res[n] > 0)
			fprintf(f, " color=red");
		if (has_map)
			fprintf(f, " shape=box");
		fprintf(f, "]\n");
	}

	gv_draw_edges(f);
	fprintf(f, "}\n");
	fclose(f);
}
